# CX52.py : CX52 v2.1, 18 July 2021, Jean-François BOUCHAUDY
import sys
import string
import cxlibs.setofwheels as cs
import cxlibs.cage as cc
import cxlibs.typewheel as ct

# ===================================
def isCapital(car):
    if string.ascii_uppercase.find(car) == -1:
        return 0
    else:
        return 1
# ===================================
def lst2str( aList ):
    return "".join( list( map( str, aList ) ))
# ===================================

class CX52:
    def __init__(self, theModel, 
            NAMEOFWHEELS, 
            ALPHA="", 
            ALPHAREV="",
            CROWN="",
            FV=False,
            KDEVICE=8,
            POSITION="",
            SLIDE=0,
            MODE='C',
            SPACE='Z'
        ):
        self.typeWheel = ct.TypeWheel(
            alphaLeft = ALPHA,
            alphaRight = ALPHAREV,
            crown = CROWN,
            FV = FV,
            mode = MODE,
            space = SPACE,
            slide = SLIDE
        )
        self.kdevice = KDEVICE
        self.nameOfWheels = NAMEOFWHEELS.split(":")
        self.nameOfModel = theModel["name"]
        self.confOfModel = theModel["conf"]
        self.existingWheels = cs.makeSetOfWheel( 
            self.nameOfModel, self.confOfModel
        )
        self.theCage = cc.Cage( 
            self.nameOfModel + "/" + self.confOfModel + ".cage"
        )
        self.theWheels = [None]*6
        for i in range(6):
            nameOfaWheel = self.nameOfWheels[i]
            if nameOfaWheel == None: 
                continue
            if nameOfaWheel in self.existingWheels:
                self.theWheels[i] = self.existingWheels[ nameOfaWheel ]
        if POSITION != "":
            posInit = POSITION.split(":")
            for i in range(6):
                if self.theWheels[i] == None: continue
                self.theWheels[i].setExternalKey( posInit[i] )

    def information(self):
        info = {}
        info["model"] = self.nameOfModel
        info["config"] = self.confOfModel
        info["cage"] = self.theCage.information()
        info["typewheel"] = self.typeWheel.information()
        info["kdevice"] = self.kdevice
        info["wheels"] = [None]*6
        for i in range(6):
            if self.theWheels[i] == None: continue
            info["wheels"][i] = self.theWheels[i].information()
        info["nameofwheels"] = self.nameOfWheels
        return info

    def code(self, letter):
        info = {}
        info["input"] = letter

        # 1) calculate the Guide
        guides, chGuides = self.getGuides()
        info["guides"] = chGuides

        # 2) calculate the advancement of wheels
        advances = self.theCage.nbOfWheelSteps( guides )
        ch=""
        for i in range(6): ch += ("%2d-"%(advances[i]))
        info["advances"] = ch

        # 3) Turn wheels
        for i in range(6):
            if self.theWheels[i] == None: continue
            self.theWheels[i].turnCounterClockWise( advances[i] )
        info["externalKey"] = self.getExtKey()

        # 4) Number of bars shifted
        nbBars = self.theCage.nbOfBarShifted( guides )
        info["nbBarsShifted"]= nbBars["shifted"]

        # 5) Kdevice
        if self.kdevice == 8:
            shift = nbBars["Kshifted"] + nbBars["Rnoshifted"] + nbBars["Tbars"]
        elif self.kdevice == 1:
            shift = nbBars["Knoshifted"] + nbBars["Rshifted"] + nbBars["Tbars"]
        else:
            if guides[ self.kdevice - 2 ]:
                shift = nbBars["Knoshifted"] + nbBars["Rshifted"] + nbBars["Tbars"]
            else:
                shift = nbBars["Kshifted"] + nbBars["Rnoshifted"] + nbBars["Tbars"]
        info["shift"] = shift

        # 6) Output
        char, charin, infotypewheel  = self.cipher(letter, shift % 26)
        info["output"] = char
        info["typewheel"] = infotypewheel

        return char, charin, info

    def cipher( self, letter, shift ):
        char, charin, info = self.typeWheel.cipher( letter, shift )
        return char, charin, info

    def getGuides(self):
        guides = []; chGuides = ""
        for i in range(6):
            if self.theWheels[i] == None:
                guides.append(None); chGuides += "-"
                continue
            guides.append( self.theWheels[i].isActive() )
            chGuides += str(self.theWheels[i].isActive() )
        return guides, chGuides

    def getExtKey(self):
        keyExt = ""
        for i in range(6):
            if self.theWheels[i] == None:
                keyExt += "  -"
                continue
            keyExt += ("%2s" % (self.theWheels[i].getExternalKey())) + "-"
        return keyExt

    def setAWheel(self, name, num):
        if name == None:
            self.theWheels[num] = None   
        elif name in self.existingWheels:
            self.theWheels[num] = self.existingWheels[ name ]

    def advanceAWheel(self, num ):
        if self.theWheels[num] == None: return
        self.theWheels[num].turnCounterClockWise( 1 )

    def gobackAWheel(self, num ):
        if self.theWheels[num] == None: return
        self.theWheels[num].turnClockWise( 1 )

if __name__ == "__main__":
    print("====> test 'CX52.py'")
    aCX = CX52({"name":"ZZ","conf":"default"}, "a29:-:-:b29:-:-")
    print( aCX.getExtKey() )        
    outPut = ""
    for char in "BON":
        sortie, inputLet, info  = aCX.code( char )
        outPut += sortie
    print("Output:", outPut )
    print( aCX.getExtKey() )        

    aCX.setAWheel( "a31", 5 )
    aCX.gobackAWheel( 5 )
    aCX.gobackAWheel( 5 )
    aCX.setAWheel( None, 0 )
    for char in "BON":
        sortie, inputLet, info  = aCX.code( char )
        outPut += sortie
        aCX.advanceAWheel(5)
        print( aCX.getExtKey() )        
    print("Output:", outPut )

